/*
Module : EZViewDoc.cpp
Purpose: Implements the MFC document class for EZView

Copyright (c) 2000 - 2017 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise)
when your product is released in binary form. You are allowed to modify the source code in any way you want
except you cannot modify the copyright details at the top of each module. If you want to distribute source
code with your application, then you are only allowed to distribute versions released by the author. This is
to maintain a single distribution point for the source code.

*/


////////////////////////// Includes ///////////////////////////////////////////

#include "stdafx.h"
#include "EZView.h"
#include "RenameDlg.h"
#include "PropertiesDlg.h"
#include "EZViewDoc.h"
#include "EZViewView.h"


////////////////////////// Macros / Defines ///////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


////////////////////////// Implementation /////////////////////////////////////

IMPLEMENT_DYNCREATE(CEZViewDoc, CDocument)

BEGIN_MESSAGE_MAP(CEZViewDoc, CDocument)
  ON_UPDATE_COMMAND_UI(ID_FILE_PRINT, &CEZViewDoc::OnUpdateFilePrint)
  ON_UPDATE_COMMAND_UI(ID_FILE_PRINT_PREVIEW, &CEZViewDoc::OnUpdateFilePrintPreview)
  ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, &CEZViewDoc::OnUpdateFileSendMail)
  ON_COMMAND(ID_FILE_RENAME, &CEZViewDoc::OnFileRename)
  ON_UPDATE_COMMAND_UI(ID_FILE_RENAME, &CEZViewDoc::OnUpdateFileRename)
  ON_COMMAND(ID_FILE_COPY, &CEZViewDoc::OnFileCopy)
  ON_UPDATE_COMMAND_UI(ID_FILE_COPY, &CEZViewDoc::OnUpdateFileCopy)
  ON_COMMAND(ID_FILE_DELETE, &CEZViewDoc::OnFileDelete)
  ON_UPDATE_COMMAND_UI(ID_FILE_DELETE, &CEZViewDoc::OnUpdateFileDelete)
  ON_COMMAND(ID_FILE_MOVE, &CEZViewDoc::OnFileMove)
  ON_UPDATE_COMMAND_UI(ID_FILE_MOVE, &CEZViewDoc::OnUpdateFileMove)
  ON_COMMAND(ID_FILE_PROPERTIES, &CEZViewDoc::OnFileProperties)
  ON_UPDATE_COMMAND_UI(ID_FILE_PROPERTIES, &CEZViewDoc::OnUpdateFileProperties)
  ON_COMMAND(ID_TOOLS_SETASWALLPAPER, &CEZViewDoc::OnToolsSetaswallpaper)
  ON_UPDATE_COMMAND_UI(ID_TOOLS_SETASWALLPAPER, &CEZViewDoc::OnUpdateToolsSetaswallpaper)
  ON_COMMAND(ID_FILE_SEND_MAIL, &CEZViewDoc::OnFileSendMail)
  ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, &CEZViewDoc::OnUpdateFileSendMail)
  ON_COMMAND(ID_VIEW_FIRSTIMAGE, &CEZViewDoc::OnViewFirstImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_FIRSTIMAGE, &CEZViewDoc::OnUpdateViewFirstImage)
  ON_COMMAND(ID_VIEW_LASTIMAGE, &CEZViewDoc::OnViewLastImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_LASTIMAGE, &CEZViewDoc::OnUpdateViewLastImage)
  ON_COMMAND(ID_VIEW_NEXTIMAGE, &CEZViewDoc::OnViewNextImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_NEXTIMAGE, &CEZViewDoc::OnUpdateViewNextImage)
  ON_COMMAND(ID_VIEW_PREVIOUSIMAGE, &CEZViewDoc::OnViewPreviousImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_PREVIOUSIMAGE, &CEZViewDoc::OnUpdateViewPreviousImage)
  ON_COMMAND(ID_VIEW_REFRESH, &CEZViewDoc::OnViewRefresh)
  ON_UPDATE_COMMAND_UI(ID_VIEW_REFRESH, &CEZViewDoc::OnUpdateViewRefresh)
  ON_COMMAND(ID_VIEW_RANDOMIMAGE, &CEZViewDoc::OnViewRandomImage)
  ON_UPDATE_COMMAND_UI(ID_VIEW_RANDOMIMAGE, &CEZViewDoc::OnUpdateViewRandomImage)
  ON_COMMAND(ID_EDIT_COPY, &CEZViewDoc::OnEditCopy)
  ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, &CEZViewDoc::OnUpdateEditCopy)
END_MESSAGE_MAP()

CEZViewDoc::CEZViewDoc()
{
}

void CEZViewDoc::Serialize(CArchive& /*ar*/)
{
  ASSERT(false); //Serialization is handled at a higher level
}

#ifdef _DEBUG
void CEZViewDoc::AssertValid() const
{
  //let the base class do its thing
  __super::AssertValid();
}

void CEZViewDoc::Dump(CDumpContext& dc) const
{
  //let the base class do its thing
  __super::Dump(dc);
}
#endif //#ifdef _DEBUG

BOOL CEZViewDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
  DeleteContents();
  ASSERT(m_Image.IsNull());

  //Construct and load the image
  HRESULT hr = m_Image.Load(lpszPathName);
  BOOL bSuccess = SUCCEEDED(hr);

  //Report any errors
  if (!bSuccess)
  {
    GetView()->KillSlideshow();
    CString sMsg;
    AfxFormatString1(sMsg, IDS_FAILED_TO_OPEN_DOC, lpszPathName);
    AfxMessageBox(sMsg);
  }
  UpdateAllViews(nullptr);

  return bSuccess;
}

void CEZViewDoc::DeleteContents() 
{
  //Destroy the image
  m_Image.Destroy();

  //Let the parent class do its thing
  __super::DeleteContents();
}

void CEZViewDoc::OnUpdateFilePrint(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnUpdateFilePrintPreview(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnUpdateFileSendMail(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnFileRename() 
{
  //Get the name of the current file
  CString sPath(GetPathName());
  TCHAR szDrive[_MAX_DRIVE];
  szDrive[0] = _T('\0');
  TCHAR szDir[_MAX_DIR];
  szDir[0] = _T('\0');
  TCHAR szFname[_MAX_FNAME];
  szFname[0] = _T('\0');
  TCHAR szExt[_MAX_EXT];
  szExt[0] = _T('\0');
  _tsplitpath_s(sPath, szDrive, sizeof(szDrive)/sizeof(TCHAR), szDir, sizeof(szDir)/sizeof(TCHAR), szFname, sizeof(szFname)/sizeof(TCHAR), szExt, sizeof(szExt)/sizeof(TCHAR));
  TCHAR szPath[_MAX_PATH];
  szPath[0] = _T('\0');
  _tmakepath_s(szPath, sizeof(szPath)/sizeof(TCHAR), nullptr, nullptr, szFname, szExt);

  //Bring up the rename dialog
  CRenameDlg dlg;
  dlg.m_sFilename = szPath;
  if (dlg.DoModal() == IDOK)
  {
    //Let the shell do the rename
    SHFILEOPSTRUCT shfo = { 0 };
    shfo.hwnd = AfxGetMainWnd()->GetSafeHwnd();
    shfo.wFunc = FO_RENAME;
    shfo.fFlags = FOF_ALLOWUNDO;
    CString sFrom(sPath);
    int nFromLength = sFrom.GetLength();
    TCHAR* pszFrom = new TCHAR[nFromLength + 2];
    _tcscpy_s(pszFrom, nFromLength + 2, sFrom);
    pszFrom[nFromLength + 1] = _T('\0');
    shfo.pFrom = pszFrom;
    _tsplitpath_s(dlg.m_sFilename, nullptr, 0, nullptr, 0, szFname, sizeof(szFname)/sizeof(TCHAR), szExt, sizeof(szExt)/sizeof(TCHAR));
    _tmakepath_s(szPath, sizeof(szPath)/sizeof(TCHAR), szDrive, szDir, szFname, szExt);
    CString sTo(szPath);
    int nToLength = sTo.GetLength();
    TCHAR* pszTo = new TCHAR[nToLength + 2];
    _tcscpy_s(pszTo, nToLength + 2, sTo);
    pszTo[nToLength + 1] = _T('\0');
    shfo.pTo = pszTo;
    int nSuccess = SHFileOperation(&shfo);
    if (nSuccess == 0)
    {
      //Set the document title to the new filename
      SetPathName(szPath);

      //Also update the image list
      theApp.RefreshImageList2(szPath);
    }

    //Free up the memory we used
    delete [] pszFrom;
    delete [] pszTo;
  }
}

void CEZViewDoc::OnUpdateFileRename(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnFileCopy() 
{
  //Get the name of the current file
  CString sPath(GetPathName());

  //Prompt the user for where they want to copy the file to
  CString sTitle;
  if (!sTitle.LoadString(IDS_FOLDER_COPY_TITLE))
    return;
  CString sDir;
  if (theApp.GetShellManager()->BrowseForFolder(sDir, AfxGetMainWnd(), nullptr, sTitle, BIF_RETURNONLYFSDIRS, nullptr))
  {
    //Ensure the directory has a "\" at its end
    int nLen = sDir.GetLength();
    if (sDir[nLen - 1] != _T('\\'))
      sDir += _T("\\");

    //Let the shell do the copy
    SHFILEOPSTRUCT shfo = { 0 };
    shfo.hwnd = AfxGetMainWnd()->GetSafeHwnd();
    shfo.wFunc = FO_COPY;
    shfo.fFlags = FOF_ALLOWUNDO;
    CString sFrom = sPath;
    int nFromLength = sFrom.GetLength();
    TCHAR* pszFrom = new TCHAR[nFromLength + 2];
    _tcscpy_s(pszFrom, nFromLength + 2, sFrom);
    pszFrom[nFromLength+1] = _T('\0');
    shfo.pFrom = pszFrom;
    TCHAR szDrive[_MAX_DRIVE];
    szDrive[0] = _T('\0');
    TCHAR szDir[_MAX_DIR];
    szDir[0] = _T('\0');
    _tsplitpath_s(sDir, szDrive, sizeof(szDrive)/sizeof(TCHAR), szDir, sizeof(szDir)/sizeof(TCHAR), nullptr, 0, nullptr, 0);
    TCHAR szFname[_MAX_FNAME];
    szFname[0] = _T('\0');
    TCHAR szExt[_MAX_EXT];
    szExt[0] = _T('\0');
    _tsplitpath_s(sPath, nullptr, 0, nullptr, 0, szFname, sizeof(szFname)/sizeof(TCHAR), szExt, sizeof(szExt)/sizeof(TCHAR));
    TCHAR szPath[_MAX_PATH];
    szPath[0] = _T('\0');
    _tmakepath_s(szPath, sizeof(szPath)/sizeof(TCHAR), szDrive, szDir, szFname, szExt);
    CString sTo(szPath);
    int nToLength = sTo.GetLength();
    TCHAR* pszTo = new TCHAR[nToLength + 2];
    _tcscpy_s(pszTo, nToLength + 2, sTo);
    pszTo[nToLength + 1] = _T('\0');
    shfo.pTo = pszTo;
    int nSuccess = SHFileOperation(&shfo);
    if (nSuccess == 0)
    {
      //Set the document title to the new filename
      SetPathName(szPath);

      //Also update the image list
      theApp.RefreshImageList2(szPath);
    }

    //Free up the memory we used
    delete [] pszFrom;
    delete [] pszTo;
  }
}

void CEZViewDoc::OnUpdateFileCopy(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnFileDelete() 
{
  //Create a Multi SZ string with the file to delete
  CString sPath(GetPathName());
  int nChars = sPath.GetLength() + 2;

  //Let the shell do the delete
  SHFILEOPSTRUCT shfo;
  CWnd* pMainWnd = AfxGetMainWnd();
  ZeroMemory(&shfo, sizeof(SHFILEOPSTRUCT));
  shfo.hwnd = pMainWnd->GetSafeHwnd();
  shfo.wFunc = FO_DELETE;

  //Undo is not allowed if the SHIFT key is held down
  if (!(GetKeyState(VK_SHIFT) & 0x8000))
    shfo.fFlags = FOF_ALLOWUNDO;

  //Should we display any UI
  if (!theApp.m_bAskMeOnDelete)
    shfo.fFlags |= FOF_NOCONFIRMATION;

  TCHAR* pszFrom = new TCHAR[nChars];
  _tcscpy_s(pszFrom, nChars, sPath);
  pszFrom[nChars - 1] = _T('\0');
  shfo.pFrom = pszFrom;

  //Let the shell perform the actual deletion
  int nSuccess = SHFileOperation(&shfo);
  if (nSuccess == 0)
  {
    //Since we have deleted the current file
    //move onto the next file in the directory
    pMainWnd->SendMessage(WM_COMMAND, ID_VIEW_NEXTIMAGE, 0L);
  }

  //Free up the memory we had allocated
  delete [] pszFrom;
}

void CEZViewDoc::OnUpdateFileDelete(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnFileMove() 
{
  //Get the name of the current file
  CString sPath(GetPathName());

  //Prompt the user for where they want to copy the file to
  CString sTitle;
  if (!sTitle.LoadString(IDS_FOLDER_COPY_TITLE))
    return;
  CString sDir;
  if (theApp.GetShellManager()->BrowseForFolder(sDir, AfxGetMainWnd(), nullptr, sTitle, BIF_RETURNONLYFSDIRS, nullptr))
  {
    //Ensure the directory has a "\" at its end
    int nLen = sDir.GetLength();
    if (sDir[nLen - 1] != _T('\\'))
      sDir += _T("\\");

    //Let the shell do the copy
    SHFILEOPSTRUCT shfo = { 0 };
    shfo.hwnd = AfxGetMainWnd()->GetSafeHwnd();
    shfo.wFunc = FO_MOVE;
    shfo.fFlags = FOF_ALLOWUNDO;
    CString sFrom = sPath;
    int nFromLength = sFrom.GetLength();
    TCHAR* pszFrom = new TCHAR[nFromLength + 2];
    _tcscpy_s(pszFrom, nFromLength + 2, sFrom);
    pszFrom[nFromLength + 1] = _T('\0');
    shfo.pFrom = pszFrom;
    TCHAR szDrive[_MAX_DRIVE];
    szDrive[0] = _T('\0');
    TCHAR szDir[_MAX_DIR];
    szDir[0] = _T('\0');
    TCHAR szFname[_MAX_FNAME];
    szFname[0] = _T('\0');
    TCHAR szExt[_MAX_EXT];
    szExt[0] = _T('\0');
    _tsplitpath_s(sDir, szDrive, sizeof(szDrive)/sizeof(TCHAR), szDir, sizeof(szDir)/sizeof(TCHAR), nullptr, 0, nullptr, 0);
    TCHAR szPath[_MAX_PATH];
    szPath[0] = _T('\0');
    _tsplitpath_s(sPath, nullptr, 0, nullptr, 0, szFname, sizeof(szFname)/sizeof(TCHAR), szExt, sizeof(szExt)/sizeof(TCHAR));
    _tmakepath_s(szPath, sizeof(szPath)/sizeof(TCHAR), szDrive, szDir, szFname, szExt);
    CString sTo(szPath);
    int nToLength = sTo.GetLength();
    TCHAR* pszTo = new TCHAR[nToLength + 2];
    _tcscpy_s(pszTo, nToLength +2, sTo);
    pszTo[nToLength + 1] = _T('\0');
    shfo.pTo = pszTo;
    int nSuccess = SHFileOperation(&shfo);
    if (nSuccess == 0)
    {
      //Set the document title to the new filename
      SetPathName(szPath);

      //Also update the image list
      theApp.RefreshImageList2(szPath);
    }

    //Free up the memory we used
    delete [] pszFrom;
    delete [] pszTo;
  }
}

void CEZViewDoc::OnUpdateFileMove(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnFileProperties() 
{
  CPropertiesDlg dlg;
  dlg.m_sFilename = GetPathName();
  dlg.m_ImageRect = CRect(0, 0, m_Image.GetWidth(), m_Image.GetHeight());
  dlg.m_nBPP = m_Image.GetBPP();
  dlg.DoModal();
}

void CEZViewDoc::OnUpdateFileProperties(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnToolsSetaswallpaper() 
{
  SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, const_cast<PVOID>(static_cast<const void*>(GetPathName().GetString())), SPIF_UPDATEINIFILE | SPIF_SENDCHANGE);
}

void CEZViewDoc::OnUpdateToolsSetaswallpaper(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(!m_Image.IsNull());
}

HGLOBAL CEZViewDoc::DDBToDIB(_In_ CBitmap* pBitmap, _In_opt_ CPalette* pPalette)
{
  //Validate our parameters
  AFXASSUME(pBitmap != nullptr);

  //Get the palette to use
  HPALETTE hPal = nullptr;
  if (pPalette)
    hPal = static_cast<HPALETTE>(pPalette->GetSafeHandle());
  if (hPal == nullptr)
    hPal = static_cast<HPALETTE>(GetStockObject(DEFAULT_PALETTE));
  
  //Get info about the DDB
  BITMAP bm = { 0 };
  pBitmap->GetObject(sizeof(bm), &bm);

  //Initialize the BITMAPINFOHEADER
  BITMAPINFOHEADER bi = { 0 };
  bi.biSize = sizeof(BITMAPINFOHEADER);
  bi.biWidth = bm.bmWidth;
  bi.biHeight = bm.bmHeight;
  bi.biPlanes = 1;
  bi.biBitCount = (WORD) (bm.bmPlanes * bm.bmBitsPixel);
  bi.biCompression = BI_RGB;

  //Compute the size of the infoheader and the color table
  int nColors = 0;
  if (bi.biBitCount <= 8)
    nColors = (1 << bi.biBitCount);
  DWORD dwLen = bi.biSize + (nColors * sizeof(RGBQUAD));

  //We need a device context to get the DDB from
  HDC hDC = GetDC(nullptr);
  if (hDC == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed in call to GetDC, Error:%u\n"), GetLastError());
    return nullptr;
  }
  hPal = SelectPalette(hDC, hPal, FALSE);
  RealizePalette(hDC);

  //Allocate enough memory to hold BITMAPINFOHEADER and color table
  LPBITMAPINFOHEADER lpBi = static_cast<LPBITMAPINFOHEADER>(GlobalAlloc(GMEM_FIXED, dwLen));
  if (lpBi == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to allocate memory for the DIB, Error:%u\n"), GetLastError());
    SelectPalette(hDC, hPal, FALSE);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  //Transfer the BITMAPINFOHEADER into the freshly allocated memory handle.
  //Note that code analysis in VS 2010 and earlier seems to have problems with the memcpy_s call below,
  //so we supress the error for that line of code
  #pragma warning(suppress: 6386)
  memcpy_s(lpBi, dwLen, &bi, dwLen);

  //Call GetDibBits with a nullptr lpBits, so the device driver will
  //calculate the biSizeImage field
  HBITMAP hBitmap = static_cast<HBITMAP>(pBitmap->GetSafeHandle());
  if (hBitmap == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to get bitmap handle. Error:%u\n"), GetLastError());
    GlobalFree(lpBi);
    SelectPalette(hDC, hPal, FALSE);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }
  GetDIBits(hDC, hBitmap, 0L, bi.biHeight, nullptr, reinterpret_cast<LPBITMAPINFO>(lpBi), DIB_RGB_COLORS);

  //Transfer back to the stack variable
  memcpy_s(&bi, sizeof(bi), lpBi, sizeof(BITMAPINFOHEADER));

  //If the driver did not fill in the biSizeImage field, then compute it.
  //Each scan line of the image is aligned on a DWORD (32 bit) boundary
  if (bi.biSizeImage == 0)
    bi.biSizeImage = ((((bi.biWidth * bi.biBitCount) + 31) & ~31) / 8) * bi.biHeight;

  //Realloc the buffer so that it can hold all the bits
  dwLen += bi.biSizeImage;
  HGLOBAL hNew = GlobalReAlloc(lpBi, dwLen, GMEM_MOVEABLE);
  if (hNew == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to realloc the memory for the DIB, Error:%u\n"), GetLastError());
    #pragma warning(suppress: 6001) //Note that code analysis in VS 2013 & 2015 seems to have problems with the GlobalFree call below so we supress the error for that line of code
    GlobalFree(lpBi);
    SelectPalette(hDC, hPal, FALSE);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  //Lock down the memory
  lpBi = static_cast<LPBITMAPINFOHEADER>(GlobalLock(hNew));
  if (lpBi == nullptr)
  {
    TRACE(_T("CEZViewDoc::DDBToDIB, Failed to lock the memory for the DIB, Error:%u\n"), GetLastError());
    GlobalFree(hNew);
    SelectPalette(hDC, hPal, FALSE);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  //Finally get the DIB
  LPBYTE lpBits = static_cast<LPBYTE>(reinterpret_cast<LPBYTE>(lpBi) + (bi.biSize + (nColors * sizeof(RGBQUAD))));
  if (!GetDIBits(hDC, hBitmap, 0, bi.biHeight, lpBits, reinterpret_cast<LPBITMAPINFO>(lpBi), DIB_RGB_COLORS))
  {
    TRACE(_T("FCEZViewDoc::DDBToDIB, Failed to get the DDB's bits, Error:%u\n"), GetLastError());
    GlobalFree(hNew);
    SelectPalette(hDC, hPal, FALSE);
    ReleaseDC(nullptr, hDC);
    return nullptr;
  }

  //Tidy up the DC before we return
  SelectPalette(hDC, hPal, FALSE);
  ReleaseDC(nullptr, hDC);

  return hNew;
}


void CEZViewDoc::OnViewFirstImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(GetPathName());

  //Now display the first image
  if (theApp.m_ImagesInCurrentFolder.size())
  {
    CString sOldPathName(GetPathName());
    CDocument* pDoc = theApp.OpenDocumentFile(0);
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate = GetDocTemplate();
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    GetView()->KillSlideshow();
    AfxMessageBox(IDS_FOLDER_DOES_NOT_CONTAIN_ANY_IMAGES);
  }
}

void CEZViewDoc::OnViewRefresh()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder
  CString sPath(GetPathName());
  theApp.RefreshImageList2(sPath);

  //Find the current image index after doing the refresh
  std::vector<CString>::iterator iterFind = std::find(theApp.m_ImagesInCurrentFolder.begin(), theApp.m_ImagesInCurrentFolder.end(), sPath);
  int nNewIndex = -1;
  if (iterFind != theApp.m_ImagesInCurrentFolder.end())
    nNewIndex = static_cast<int>(std::distance(theApp.m_ImagesInCurrentFolder.begin(), iterFind));

  //Redisplay this image
  CString sOldPathName(GetPathName());
  CDocTemplate* pTemplate = GetDocTemplate();
  AFXASSUME(pTemplate != nullptr);
  pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
  CDocument* pDoc = nullptr;
  if (nNewIndex != -1)
    pDoc = theApp.OpenDocumentFile(nNewIndex);
  if (pDoc == nullptr)
  {
    //Reset this document to the current image before the failed load above
    if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
      pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
  }
}

void CEZViewDoc::OnUpdateViewRefresh(CCmdUI* pCmdUI)
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnViewLastImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath(GetPathName());
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Now display the last image
  int nSize = static_cast<int>(theApp.m_ImagesInCurrentFolder.size());
  if (nSize)
  {
    CString sOldPathName(GetPathName());
    CDocument* pDoc = theApp.OpenDocumentFile(nSize - 1);
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate = GetDocTemplate();
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    GetView()->KillSlideshow();
    AfxMessageBox(IDS_FOLDER_DOES_NOT_CONTAIN_ANY_IMAGES);
  }
}

void CEZViewDoc::OnViewNextImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath(GetPathName());
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Find the current image index if unknown
  if (theApp.m_nCurrentImageIndex == -1)
  {
    std::vector<CString>::iterator iterFind = std::find(theApp.m_ImagesInCurrentFolder.begin(), theApp.m_ImagesInCurrentFolder.end(), sCurrentPath);
    if (iterFind != theApp.m_ImagesInCurrentFolder.end())
      theApp.m_nCurrentImageIndex = static_cast<int>(std::distance(theApp.m_ImagesInCurrentFolder.begin(), iterFind));
  }

  //If we found this image then display the next one
  if (theApp.m_nCurrentImageIndex < static_cast<int>(theApp.m_ImagesInCurrentFolder.size() - 1))
  {
    CString sOldPathName(GetPathName());
    CDocument* pDoc = theApp.OpenNextDocumentFile();
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate = GetDocTemplate();
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    //We've reached the last image, ask if they want to go back to the begining
    bool bShowFirst = false;
    if (theApp.m_bPromptStartEndFolder)
    {
      GetView()->KillSlideshow();

      //Ask the user if we can go the last image
      CString sContent;
      if (!sContent.LoadString(IDS_REACHED_LAST_IMAGE_IN_FOLDER))
        return;
      CTaskDialog dlg(sContent, _T(""), theApp.m_pszAppName, TDCBF_YES_BUTTON | TDCBF_NO_BUTTON);
      dlg.SetOptions(TDF_ALLOW_DIALOG_CANCELLATION | TDF_POSITION_RELATIVE_TO_WINDOW);
      CString sDontAskAgain;
      if (!sDontAskAgain.LoadString(IDS_DONT_ASK_AGAIN))
        return;
      dlg.SetVerificationCheckboxText(sDontAskAgain);
      bShowFirst = (dlg.DoModal(AfxGetMainWnd()->GetSafeHwnd()) == IDYES);
      theApp.m_bPromptStartEndFolder = !dlg.GetVerificationCheckboxState();
    }
    else
      bShowFirst = true;

    if (bShowFirst)
      OnViewFirstImage();
  }
}

void CEZViewDoc::OnViewPreviousImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath(GetPathName());
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Find the current image index if unknown
  if (theApp.m_nCurrentImageIndex == -1)
  {
    std::vector<CString>::iterator iterFind = std::find(theApp.m_ImagesInCurrentFolder.begin(), theApp.m_ImagesInCurrentFolder.end(), sCurrentPath);
    if (iterFind != theApp.m_ImagesInCurrentFolder.end())
      theApp.m_nCurrentImageIndex = static_cast<int>(std::distance(theApp.m_ImagesInCurrentFolder.begin(), iterFind));
  }

  //Then display the previous one
  if (theApp.m_nCurrentImageIndex > 0)
  {
    CString sOldPathName(GetPathName());
    CDocument* pDoc = theApp.OpenPrevDocumentFile();
    if (pDoc == nullptr)
    {
      //Reset this document to the current image before the failed load above
      CDocTemplate* pTemplate = GetDocTemplate();
      AFXASSUME(pTemplate != nullptr);
      if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
        pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
    }
  }
  else
  {
    //We've reached the first image, ask if they want to go back to the end
    bool bShowLast = false;
    if (theApp.m_bPromptStartEndFolder)
    {
      GetView()->KillSlideshow();

      CString sContent;
      if (!sContent.LoadString(IDS_REACHED_FIRST_IMAGE_IN_FOLDER))
        return;
      CTaskDialog dlg(sContent, _T(""), theApp.m_pszAppName, TDCBF_YES_BUTTON | TDCBF_NO_BUTTON);
      dlg.SetOptions(TDF_ALLOW_DIALOG_CANCELLATION | TDF_POSITION_RELATIVE_TO_WINDOW);
      CString sDontAskAgain;
      if (!sDontAskAgain.LoadString(IDS_DONT_ASK_AGAIN))
        return;
      dlg.SetVerificationCheckboxText(sDontAskAgain);
      bShowLast = (dlg.DoModal(AfxGetMainWnd()->GetSafeHwnd()) == IDYES);
      theApp.m_bPromptStartEndFolder = !dlg.GetVerificationCheckboxState();
    }
    else
      bShowLast = true;

    if (bShowLast)
      OnViewLastImage();
  }
}

void CEZViewDoc::OnViewRandomImage()
{
  //Display the hourglass as this may take some time
  CWaitCursor wait;

  //Refresh the image list for this folder if it is currently empty
  CString sCurrentPath(GetPathName());
  if (theApp.m_ImagesInCurrentFolder.size() == 0)
    theApp.RefreshImageList2(sCurrentPath);

  //Work out the next image to show
  int nNewIndex = rand() % theApp.m_ImagesInCurrentFolder.size();
  CString sOldPathName(GetPathName());
  CDocument* pDoc = theApp.OpenDocumentFile(nNewIndex);
  if (pDoc == nullptr)
  {
    //Reset this document to the current image before the failed load above
    CDocTemplate* pTemplate = GetDocTemplate();
    AFXASSUME(pTemplate != nullptr);
    if (pTemplate->OpenDocumentFile(sOldPathName, FALSE, TRUE) == nullptr)
      pTemplate->OpenDocumentFile(nullptr, FALSE, TRUE);
  }
}

void CEZViewDoc::OnUpdateViewRandomImage(CCmdUI* pCmdUI)
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnUpdateViewFirstImage(CCmdUI* pCmdUI)
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnUpdateViewLastImage(CCmdUI* pCmdUI)
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnUpdateViewNextImage(CCmdUI* pCmdUI)
{
  pCmdUI->Enable(!m_Image.IsNull());
}

void CEZViewDoc::OnUpdateViewPreviousImage(CCmdUI* pCmdUI)
{
  pCmdUI->Enable(!m_Image.IsNull());
}

CEZViewView* CEZViewDoc::GetView() const
{
  POSITION posView = GetFirstViewPosition();
  while (posView != nullptr)
  { 
    CEZViewView* pView = static_cast<CEZViewView*>(GetNextView(posView));
    ASSERT(pView != nullptr);
    return pView;
  }
  return nullptr;
}


void CEZViewDoc::OnEditCopy()
{
  //Copy from the DDB to DIB and transfer to the clipboard
  CBitmap bitmap;
  bitmap.Attach(m_Image.operator HBITMAP());
  HGLOBAL hDib = DDBToDIB(&bitmap, nullptr);
  if (hDib && GetView()->OpenClipboard())
  {
    EmptyClipboard();
    SetClipboardData(CF_DIB, hDib);
    CloseClipboard();
  }
  bitmap.Detach();
}

void CEZViewDoc::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
  pCmdUI->Enable(!m_Image.IsNull());
}
