/*
Module : EZViewDoc.h
Purpose: Defines the MFC document class for EZView

Copyright (c) 2000 - 2017 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise)
when your product is released in binary form. You are allowed to modify the source code in any way you want
except you cannot modify the copyright details at the top of each module. If you want to distribute source
code with your application, then you are only allowed to distribute versions released by the author. This is
to maintain a single distribution point for the source code.

*/


////////////////////////// Macros / Defines ///////////////////////////////////

#pragma once


////////////////////////// Classes ////////////////////////////////////////////

class CEZViewView;

class CEZViewDoc : public CDocument
{
public:
//Methods
#ifdef _DEBUG
  virtual void AssertValid() const;
  virtual void Dump(CDumpContext& dc) const;
#endif //#ifdef _DEBUG
  HGLOBAL DDBToDIB(_In_ CBitmap* pBitmap, _In_opt_ CPalette* pPalette);

//Member variables
  ATL::CImage m_Image;

protected:
  CEZViewDoc(); //create from serialization only

//Methods
  virtual void Serialize(CArchive& ar);
  virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
  virtual void DeleteContents();
  CEZViewView* GetView() const;

//Message handlers
  afx_msg void OnUpdateFilePrint(CCmdUI* pCmdUI);
  afx_msg void OnUpdateFilePrintPreview(CCmdUI* pCmdUI);
  afx_msg void OnUpdateFileSendMail(CCmdUI* pCmdUI);
  afx_msg void OnFileRename();
  afx_msg void OnUpdateFileRename(CCmdUI* pCmdUI);
  afx_msg void OnFileCopy();
  afx_msg void OnUpdateFileCopy(CCmdUI* pCmdUI);
  afx_msg void OnFileDelete();
  afx_msg void OnUpdateFileDelete(CCmdUI* pCmdUI);
  afx_msg void OnFileMove();
  afx_msg void OnUpdateFileMove(CCmdUI* pCmdUI);
  afx_msg void OnFileProperties();
  afx_msg void OnUpdateFileProperties(CCmdUI* pCmdUI);
  afx_msg void OnToolsSetaswallpaper();
  afx_msg void OnUpdateToolsSetaswallpaper(CCmdUI* pCmdUI);
  afx_msg void OnViewFirstImage();
  afx_msg void OnUpdateViewFirstImage(CCmdUI* pCmdUI);
  afx_msg void OnViewLastImage();
  afx_msg void OnUpdateViewLastImage(CCmdUI* pCmdUI);
  afx_msg void OnViewNextImage();
  afx_msg void OnUpdateViewNextImage(CCmdUI* pCmdUI);
  afx_msg void OnViewPreviousImage();
  afx_msg void OnUpdateViewPreviousImage(CCmdUI* pCmdUI);
  afx_msg void OnViewRefresh();
  afx_msg void OnUpdateViewRefresh(CCmdUI* pCmdUI);
  afx_msg void OnViewRandomImage();
  afx_msg void OnUpdateViewRandomImage(CCmdUI* pCmdUI);
  afx_msg void OnEditCopy();
  afx_msg void OnUpdateEditCopy(CCmdUI* pCmdUI);

  DECLARE_DYNCREATE(CEZViewDoc)

  DECLARE_MESSAGE_MAP()
};
