/*
Module : EZViewDoc.cpp
Purpose: Implements the MFC document class for EZView

Copyright (c) 2000 - 2017 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise)
when your product is released in binary form. You are allowed to modify the source code in any way you want
except you cannot modify the copyright details at the top of each module. If you want to distribute source
code with your application, then you are only allowed to distribute versions released by the author. This is
to maintain a single distribution point for the source code.

*/


////////////////////////// Includes ///////////////////////////////////////////

#include "stdafx.h"
#include "EZView.h"
#include "EZViewDoc.h"
#include "EZViewView.h"


////////////////////////// Macros / Defines ///////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


////////////////////////// Implementation /////////////////////////////////////

IMPLEMENT_DYNCREATE(CEZViewView, CScrollView)

BEGIN_MESSAGE_MAP(CEZViewView, CScrollView)
  ON_WM_ERASEBKGND()
  ON_WM_CONTEXTMENU()
  ON_COMMAND(ID_ZOOM_ZOOMIN, &CEZViewView::OnZoomZoomin)
  ON_COMMAND(ID_ZOOM_ZOOMOUT, &CEZViewView::OnZoomZoomout)
  ON_UPDATE_COMMAND_UI(ID_ZOOM_ZOOMIN, &CEZViewView::OnUpdateZoomZoomin)
  ON_UPDATE_COMMAND_UI(ID_ZOOM_ZOOMOUT, &CEZViewView::OnUpdateZoomZoomout)
  ON_COMMAND(ID_ZOOM_ACTUAL, &CEZViewView::OnZoomActual)
  ON_UPDATE_COMMAND_UI(ID_ZOOM_ACTUAL, &CEZViewView::OnUpdateZoomActual)
  ON_COMMAND(ID_ZOOM_BESTFIT, &CEZViewView::OnZoomBestfit)
  ON_UPDATE_COMMAND_UI(ID_ZOOM_BESTFIT, &CEZViewView::OnUpdateZoomBestfit)
  ON_COMMAND(ID_TOOLS_SLIDESHOW_RUN, &CEZViewView::OnToolsSlideshowRun)
  ON_UPDATE_COMMAND_UI(ID_TOOLS_SLIDESHOW_RUN, &CEZViewView::OnUpdateToolsSlideshowRun)
  ON_WM_TIMER()
  ON_WM_DESTROY()
  ON_COMMAND(ID_FILE_PRINT, &CScrollView::OnFilePrint)
  ON_COMMAND(ID_FILE_PRINT_DIRECT, &CScrollView::OnFilePrint)
  ON_COMMAND(ID_FILE_PRINT_PREVIEW, &CScrollView::OnFilePrintPreview)
  ON_MESSAGE(WM_APPCOMMAND, &CEZViewView::OnAppCommand)
  ON_WM_LBUTTONDBLCLK()
END_MESSAGE_MAP()

CEZViewView::CEZViewView() : m_nCurrentZoomLevel(100),
                             m_bSizeToFit(false),
                             m_nTimerID(0)
{
}

CSize CEZViewView::DrawSize(_In_ CEZViewDoc* pDoc, _In_ CRect rClient)
{
  //Validate our parameters
  AFXASSUME(pDoc != nullptr);

  int nImageWidth = pDoc->m_Image.GetWidth();
  int nImageHeight = pDoc->m_Image.GetHeight();
  if (m_bSizeToFit)
  {
    if ((nImageWidth < rClient.Width()) && (nImageHeight < rClient.Height()))
    {
      m_nCurrentZoomLevel = rClient.Width() * 100 / nImageWidth;
      int nZoom = rClient.Height() * 100 / nImageHeight;
      if (nZoom < m_nCurrentZoomLevel)
        m_nCurrentZoomLevel = nZoom; 
    }
    else if ((nImageWidth > rClient.Width()) || (nImageHeight > rClient.Height()))
    {
      m_nCurrentZoomLevel = rClient.Width() * 100 / nImageWidth;
      int nZoom = rClient.Height() * 100 / nImageHeight;
      if (nZoom < m_nCurrentZoomLevel)
        m_nCurrentZoomLevel = nZoom; 
    }
  }

  return CSize(nImageWidth * m_nCurrentZoomLevel / 100, nImageHeight * m_nCurrentZoomLevel / 100);
}

void CEZViewView::OnDraw(CDC* pDC)
{
  CEZViewDoc* pDoc = GetDocument();
  AFXASSUME(pDoc != nullptr);

  bool bFullscreen = GetMainFrame()->FullScreen();

  //Draw the image if the document has one
  if (!pDoc->m_Image.IsNull())
  {
    //Get the client rect
    CRect r;
    GetClientRect(&r);

    //Determine where the image should be drawn
    CSize drawSize = DrawSize(pDoc, r);
    CRect rDraw(0, 0, drawSize.cx, drawSize.cy);
    if (bFullscreen)
    {
      if (drawSize.cx < r.Width())
        rDraw.left = (r.Width() - drawSize.cx) / 2;
      if (drawSize.cy < r.Height())
        rDraw.top = (r.Height() - drawSize.cy) / 2;
      rDraw.right = rDraw.left + drawSize.cx;
      rDraw.bottom = rDraw.top + drawSize.cy;
    }

    //Render the image
    pDoc->m_Image.Draw(pDC->GetSafeHdc(), rDraw);

    //Draw the filename if configured to do so
    if (bFullscreen && theApp.m_bShowFilenameDuringFullScreen)
    {
      //Setup the DC
      int nOldMode = pDC->SetBkMode(TRANSPARENT);
      int nOldColor = pDC->SetTextColor(theApp.m_FullScreenTextColor);
      CFont defaultGUIFont;
      defaultGUIFont.Attach(GetStockObject(DEFAULT_GUI_FONT));
      HGDIOBJ hOldObj = pDC->SelectObject(&defaultGUIFont);

      //Form the text to display
      CString sFilename;
      if (theApp.m_nCurrentImageIndex != -1)
      {
        CString sCurrent;
        sCurrent.Format(_T("%d"), theApp.m_nCurrentImageIndex + 1);
        CString sTotal;
        sTotal.Format(_T("%d"), static_cast<int>(theApp.m_ImagesInCurrentFolder.size()));
        CString sPosition;
        AfxFormatString2(sPosition, IDS_INDICATOR_IMAGE, sCurrent, sTotal);
        AfxFormatString2(sFilename, IDS_FULLSCREEN_CAPTION, pDoc->GetPathName(), sPosition);
      }
      else
        sFilename = pDoc->GetPathName();

      //Do the actual drawing
      CSize size = pDC->GetTextExtent(sFilename);
      r.top = 5;
      r.bottom = r.top + size.cy;
      r.left = 5;
      r.right = r.left + size.cx;
      pDC->DrawText(sFilename, &r, DT_LEFT);

      //Restore the DC
      pDC->SelectObject(hOldObj);
      pDC->SetTextColor(nOldColor);
      pDC->SetBkMode(nOldMode);
    }
  }
}

BOOL CEZViewView::OnPreparePrinting(CPrintInfo* pInfo)
{
  // default preparation
  return DoPreparePrinting(pInfo);
}

void CEZViewView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CEZViewView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

#ifdef _DEBUG
void CEZViewView::AssertValid() const
{
  //Let the base class do its thing
  __super::AssertValid();
}

void CEZViewView::Dump(CDumpContext& dc) const
{
  //Let the base class do its thing
  __super::Dump(dc);
}
#endif //#ifdef _DEBUG

CEZViewDoc* CEZViewView::GetDocument()
{
  ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CEZViewDoc)));
  return static_cast<CEZViewDoc*>(m_pDocument);
}

void CEZViewView::OnUpdate(CView* /*pSender*/, LPARAM /*lHint*/, CObject* /*pHint*/) 
{
  CEZViewDoc* pDoc = GetDocument();
  AFXASSUME(pDoc != nullptr);

  //Set the scroll view extend to that of the picture in the document
  if (!pDoc->m_Image.IsNull())
  {
    CRect r;
    GetClientRect(&r);
    SetScrollSizes(MM_TEXT, DrawSize(pDoc, r));
  }
  else
    SetScrollSizes(MM_TEXT, CSize(0, 0));
  RedrawWindow();
}

BOOL CEZViewView::OnEraseBkgnd(CDC* pDC) 
{
  //Draw the background
  if (GetMainFrame()->FullScreen())
  {
    CRect r;
    GetClientRect(&r);
    pDC->FillSolidRect(r, theApp.m_FullScreenBackgroundColor);
  }
  else
  {
    CBrush br;
    br.CreateHatchBrush(HS_DIAGCROSS, RGB(192, 192, 192));
    FillOutsideRect(pDC, &br);
  }

  return TRUE;
}

BOOL CEZViewView::PreTranslateMessage(MSG* pMsg) 
{
  if ((((pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN) && // If we hit a key and
      (pMsg->wParam == VK_F10) && (GetKeyState(VK_SHIFT) & ~1)) != 0) ||	 // it's Shift+F10 OR
      (pMsg->message == WM_CONTEXTMENU))									                 // Natural keyboard key
  {
    CPoint point;
    GetCursorPos(&point);
    OnContextMenu(nullptr, point);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_DOWN)
  {
    SendMessage(WM_VSCROLL, SB_LINEDOWN);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_UP)
  {
    SendMessage(WM_VSCROLL, SB_LINEUP);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_LEFT)
  {
    SendMessage(WM_HSCROLL, SB_LINEUP);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_RIGHT)
  {
    SendMessage(WM_HSCROLL, SB_LINEDOWN);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_NEXT)
  {
    SendMessage(WM_VSCROLL, SB_PAGEDOWN);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_PRIOR)
  {
    SendMessage(WM_VSCROLL, SB_PAGEUP);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_HOME)
  {
    SendMessage(WM_HSCROLL, SB_PAGEUP);
    return TRUE;
  }
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_END)
  {
    SendMessage(WM_HSCROLL, SB_PAGEDOWN);
    return TRUE;
  }
  else
  {
    //Let the base class do its thing
    return __super::PreTranslateMessage(pMsg);
  }
}

void CEZViewView::OnContextMenu(CWnd* /*pWnd*/, CPoint point) 
{
  CMenu menu;
  if (!menu.LoadMenu(IDR_POPUP_VIEW))
    return;

  CMenu* pPopup = menu.GetSubMenu(0);
  AFXASSUME(pPopup != nullptr);

  CWnd* pWndPopupOwner = this;
  while (pWndPopupOwner->GetStyle() & WS_CHILD)
    pWndPopupOwner = pWndPopupOwner->GetParent();

  pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, pWndPopupOwner);
}

void CEZViewView::OnZoomZoomin() 
{
  CEZViewDoc* pDoc = GetDocument();
  AFXASSUME(pDoc != nullptr);

  m_nCurrentZoomLevel = (m_nCurrentZoomLevel/20*20) + 20;
  m_bSizeToFit = false;
  CRect r;
  GetClientRect(&r);
  SetScrollSizes(MM_TEXT, DrawSize(pDoc, r));
  RedrawWindow();
}

void CEZViewView::OnZoomZoomout() 
{
  CEZViewDoc* pDoc = GetDocument();
  AFXASSUME(pDoc != nullptr);

  m_nCurrentZoomLevel = (m_nCurrentZoomLevel/20*20) - 20;
  m_bSizeToFit = false;
  CRect r;
  GetClientRect(&r);
  SetScrollSizes(MM_TEXT, DrawSize(pDoc, r));
  RedrawWindow();
}

void CEZViewView::OnUpdateZoomZoomin(CCmdUI* pCmdUI) 
{
  if (!GetDocument()->m_Image.IsNull())
    pCmdUI->Enable(m_bSizeToFit || (m_nCurrentZoomLevel < 300));
  else
    pCmdUI->Enable(FALSE);
}

void CEZViewView::OnUpdateZoomZoomout(CCmdUI* pCmdUI) 
{
  if (!GetDocument()->m_Image.IsNull())
    pCmdUI->Enable(m_bSizeToFit || (m_nCurrentZoomLevel > 20));
  else
    pCmdUI->Enable(FALSE);
}

void CEZViewView::OnZoomActual() 
{
  CEZViewDoc* pDoc = GetDocument();
  AFXASSUME(pDoc != nullptr);

  m_nCurrentZoomLevel = 100;
  m_bSizeToFit = false;
  CRect r;
  GetClientRect(&r);
  SetScrollSizes(MM_TEXT, DrawSize(pDoc, r));
  RedrawWindow();
}

void CEZViewView::OnUpdateZoomActual(CCmdUI* pCmdUI) 
{
  if (!GetDocument()->m_Image.IsNull())
    pCmdUI->Enable(m_bSizeToFit || (m_nCurrentZoomLevel != 100));
  else
    pCmdUI->Enable(FALSE);
}

void CEZViewView::OnZoomBestfit() 
{
  CEZViewDoc* pDoc = GetDocument();
  AFXASSUME(pDoc != nullptr);

  m_bSizeToFit = !m_bSizeToFit;	
  CRect r;
  GetClientRect(&r);
  SetScrollSizes(MM_TEXT, DrawSize(pDoc, r));
  RedrawWindow();
}

void CEZViewView::OnUpdateZoomBestfit(CCmdUI* pCmdUI) 
{
  if (!GetDocument()->m_Image.IsNull())
    pCmdUI->SetCheck(m_bSizeToFit);
  else
    pCmdUI->Enable(FALSE);
}

void CEZViewView::OnToolsSlideshowRun() 
{
  if (m_nTimerID == 0)
  {
    //Set the timer to fire
    m_nTimerID = SetTimer(1, theApp.m_nSlideshowInterval*1000, nullptr);
  }
  else
    KillSlideshow();
}

void CEZViewView::KillSlideshow()
{
  //Stop slideshow if active
  if (m_nTimerID != 0)
  {
    KillTimer(m_nTimerID);
    m_nTimerID = 0;
  }
}
 
void CEZViewView::OnUpdateToolsSlideshowRun(CCmdUI* pCmdUI) 
{
  if (!GetDocument()->m_Image.IsNull())
    pCmdUI->SetCheck(m_nTimerID != 0);
  else
    pCmdUI->Enable(FALSE);
}

void CEZViewView::OnTimer(UINT_PTR nIDEvent) 
{
  if ((m_nTimerID == nIDEvent) && !theApp.m_bMessageBoxUp)
  {
    //Go to the next message
    AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_VIEW_NEXTIMAGE);
  }
  else	
  {
    //Let the base class do its thing
    __super::OnTimer(nIDEvent);
  }
}

void CEZViewView::OnDestroy() 
{
  KillSlideshow();

  //Let the base class do its thing
  __super::OnDestroy();
}

LRESULT CEZViewView::OnAppCommand(WPARAM /*wParam*/, LPARAM lParam)
{
  bool bHandled = false;
  short cmd = GET_APPCOMMAND_LPARAM(lParam);
  switch (cmd)
  {
    case APPCOMMAND_BROWSER_FORWARD:
    case APPCOMMAND_MEDIA_NEXTTRACK:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_VIEW_NEXTIMAGE);
      break;
    }
    case APPCOMMAND_BROWSER_REFRESH:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_VIEW_REFRESH);
      break;
    }
    case APPCOMMAND_BROWSER_BACKWARD:
    case APPCOMMAND_MEDIA_PREVIOUSTRACK:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_VIEW_PREVIOUSIMAGE);
      break;
    }
    case APPCOMMAND_MEDIA_PLAY_PAUSE:
    {
      bHandled = true;
      if (!GetDocument()->m_Image.IsNull())
        OnToolsSlideshowRun();
      break;
    }
    case APPCOMMAND_MEDIA_PLAY:
    {
      bHandled = true;
      if (!GetDocument()->m_Image.IsNull() && (m_nTimerID == 0))
        OnToolsSlideshowRun();
      break;
    }
    case APPCOMMAND_MEDIA_STOP:
    case APPCOMMAND_BROWSER_STOP:
    {
      bHandled = true;
      if (!GetDocument()->m_Image.IsNull() && (m_nTimerID != 0))
        OnToolsSlideshowRun();
      break;
    }
    case APPCOMMAND_MEDIA_REWIND:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_VIEW_FIRSTIMAGE);
      break;
    }
    case APPCOMMAND_CLOSE:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_CLOSE);
      break;
    }
    case APPCOMMAND_SEND_MAIL:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_FILE_SEND_MAIL);
      break;
    }
    case APPCOMMAND_HELP:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_HELP_INDEX);
      break;
    }
    case APPCOMMAND_OPEN:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_FILE_OPEN);
      break;
    }
    case APPCOMMAND_PRINT:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_FILE_PRINT);
      break;
    }
    case APPCOMMAND_COPY:
    {
      bHandled = true;
      AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_EDIT_COPY);
      break;
    }
    default:
    {
      break;
    }
  }

  if (!bHandled)
    return Default();
  else
    return 0L;
}

void CEZViewView::OnLButtonDblClk(UINT nFlags, CPoint point)
{
  //A double click goes in or out of fullscreen mode
  AfxGetMainWnd()->SendMessage(WM_COMMAND, ID_VIEW_FULLSCREEN);

  //Let the base class do its thing
  __super::OnLButtonDblClk(nFlags, point);
}
