/*
Module : PropertiesDlg.cpp
Purpose: Implements the properties dialog for EZView

Copyright (c) 2000 - 2017 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise)
when your product is released in binary form. You are allowed to modify the source code in any way you want
except you cannot modify the copyright details at the top of each module. If you want to distribute source
code with your application, then you are only allowed to distribute versions released by the author. This is
to maintain a single distribution point for the source code.

*/


////////////////////////// Includes ///////////////////////////////////////////

#include "stdafx.h"
#include "EZView.h"
#include "PropertiesDlg.h"


////////////////////////// Macros / Defines ///////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


////////////////////////// Implementation /////////////////////////////////////

CPropertiesDlg::CPropertiesDlg(_In_opt_ CWnd* pParent) : CDialog(CPropertiesDlg::IDD, pParent)
{
}

void CPropertiesDlg::DoDataExchange(CDataExchange* pDX)
{
  //Let the base class do its thing
  __super::DoDataExchange(pDX);

  DDX_Control(pDX, IDC_BITSPERPIXEL, m_ctrlBitsPerPixel);
  DDX_Control(pDX, IDC_MODIFIED, m_ctrlModified);
  DDX_Control(pDX, IDC_CREATED, m_ctrlCreated);
  DDX_Control(pDX, IDC_ACCESSED, m_ctrlAccessed);
  DDX_Control(pDX, IDC_FILESIZE, m_ctrlSize);
  DDX_Control(pDX, IDC_FILENAME, m_ctrlFilename);
  DDX_Control(pDX, IDC_DIMENSIONS, m_ctrlDimensions);
  DDX_Text(pDX, IDC_FILENAME, m_sFilename);
}

BEGIN_MESSAGE_MAP(CPropertiesDlg, CDialog)
END_MESSAGE_MAP()


BOOL CPropertiesDlg::OnInitDialog() 
{
  //Let the parent class do its thing
  __super::OnInitDialog();
  
  //Set the various text 
  CFileStatus fs;
  if (CFile::GetStatus(m_sFilename, fs))
  {
    m_ctrlCreated.SetWindowText(FormatDateTime(fs.m_ctime));
    m_ctrlAccessed.SetWindowText(FormatDateTime(fs.m_atime));
    m_ctrlModified.SetWindowText(FormatDateTime(fs.m_mtime));
    m_ctrlSize.SetWindowText(FormatBytes(fs.m_size));
    m_ctrlDimensions.SetWindowText(FormatDimensions());
    m_ctrlBitsPerPixel.SetWindowText(PixelFormat());
  }
  
  return TRUE;
}

CString CPropertiesDlg::FormatDateTime(_In_ const CTime& time)
{
  COleDateTime oleTime(time.GetYear(), time.GetMonth(), time.GetDay(), time.GetHour(), time.GetMinute(), time.GetSecond());
  return oleTime.Format();
}

CString CPropertiesDlg::PixelFormat()
{
  CString sText;
  sText.Format(_T("%d"), m_nBPP);
  return sText;
}

CString CPropertiesDlg::FormatDimensions()
{
  CString sText;
  sText.Format(_T("%d x %d"), m_ImageRect.Width(), m_ImageRect.Height());
  return sText;
}

CString CPropertiesDlg::FormatBytes(_In_ ULONGLONG dwFileSize)
{
  CString sBytes;
  sBytes.Format(_T("%I64u"), dwFileSize);

  CString sFirst;
  if (dwFileSize < 1024)
    AfxFormatString1(sFirst, IDS_PROPERTIES_BYTES, sBytes);
  else if (dwFileSize < 1048576)
  {
    CString sKiloBytes;
    sKiloBytes.Format(_T("%0.1f"), dwFileSize/1024.0);
    AfxFormatString1(sFirst, IDS_PROPERTIES_KILOBYTES, sKiloBytes);
  }
  else
  {
    CString sMegaBytes;
    sMegaBytes.Format(_T("%0.2f"), dwFileSize/1048576.0);
    AfxFormatString1(sFirst, IDS_PROPERTIES_MEGABYTES, sMegaBytes);
  }

  CString sText;
  AfxFormatString2(sText, IDS_PROPERTIES_SIZE, sFirst, sBytes);
  return sText;
}  
